from __future__ import annotations

from .. import core as core
from .. import framework as framework
from ..framework import OpProtoHolder as OpProtoHolder
from ..framework import Variable as Variable
from ..framework import convert_np_dtype_to_dtype_ as convert_np_dtype_to_dtype_
from ..framework import in_dygraph_mode as in_dygraph_mode
from . import no_grad as no_grad

def monkey_patch_math_varbase(): ...



from __future__ import annotations

from typing import Any

from paddle.distributed.fleet.meta_optimizers.common import (
    OP_ROLE_VAR_KEY as OP_ROLE_VAR_KEY,
)
from paddle.fluid.framework import Parameter as Parameter
from paddle.fluid.framework import Program as Program
from paddle.fluid.framework import Variable as Variable
from paddle.fluid.framework import program_guard as program_guard

from ..dist_attribute import (
    OperatorDistributedAttribute as OperatorDistributedAttribute,
)
from ..process_group import new_process_group as new_process_group
from ..utils import (
    compute_compatible_and_update_dim_mapping as compute_compatible_and_update_dim_mapping,
)
from ..utils import compute_compatible_dim_mapping as compute_compatible_dim_mapping
from ..utils import compute_compatible_dims_mapping as compute_compatible_dims_mapping
from ..utils import is_dim_replicate as is_dim_replicate
from ..utils import is_dim_shard as is_dim_shard
from ..utils import is_valid_list_index as is_valid_list_index
from ..utils import set_dist_op_desc_original_id as set_dist_op_desc_original_id
from .common import DistributedOperatorImpl as DistributedOperatorImpl
from .common import DistributedOperatorImplContainer as DistributedOperatorImplContainer
from .common import infer_shape as infer_shape
from .common import is_parameter_related as is_parameter_related
from .common import (
    naive_copy_op_dist_attr_for_program as naive_copy_op_dist_attr_for_program,
)
from .common import (
    register_distributed_operator_impl as register_distributed_operator_impl,
)
from .common import (
    register_distributed_operator_impl_container as register_distributed_operator_impl_container,
)
from .common import (
    set_comm_op_dist_attr_for_program as set_comm_op_dist_attr_for_program,
)
from .dist_default import DistributedDefaultImpl0 as DistributedDefaultImpl0

def copy_op_with_new_input_output(ctx: Any, block: Any, src_op: Any, **kwargs: Any): ...

class DistributedMatmul(DistributedOperatorImplContainer):
    def __init__(self, op_type: Any) -> None: ...

class DistributedMatmulImpl0(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMatmulImpl1(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMatmulImpl2(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMatmulV2(DistributedOperatorImplContainer):
    def __init__(self, op_type: Any) -> None: ...

class DistributedMatmulV2Impl0(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMatmulV2Impl1(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMatmulV2Impl2(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMul(DistributedOperatorImplContainer):
    def __init__(self, op_type: Any) -> None: ...

class DistributedMulImpl0(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMulImpl1(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...

class DistributedMulImpl2(DistributedOperatorImpl):
    def __init__(self, name: Any) -> None: ...
    def is_input_compatible(self, dist_op: Any): ...
    def is_output_compatible(self, dist_op: Any): ...
    def is_auto_compatible(self, dist_op: Any): ...
    def update_dims_mapping(self, dist_op: Any): ...
    @staticmethod
    def forward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...
    @staticmethod
    def backward(ctx: Any, *args: Any, **kwargs: Any) -> None: ...



from __future__ import annotations

from typing import Callable

from typing_extensions import ParamSpec, TypeVar

InputArgs = ParamSpec("InputArgs")
RetValue = TypeVar("RetValue")

from .._typing import CPUPlace as CPUPlace
from .._typing import CUDAPinnedPlace as CUDAPinnedPlace
from .._typing import CUDAPlace as CUDAPlace
from .._typing import CustomPlace as CustomPlace
from .._typing import IPUPlace as IPUPlace
from .._typing import MLUPlace as MLUPlace
from .._typing import NPUPlace as NPUPlace
from ..fluid import core as core
from ..fluid.core import VarBase as VarBase
from ..fluid.dygraph.base import grad as grad
from ..fluid.dygraph.parallel import DataParallel as DataParallel
from ..fluid.framework import OpProtoHolder as OpProtoHolder
from ..fluid.framework import convert_np_dtype_to_dtype_ as convert_np_dtype_to_dtype_
from ..fluid.framework import disable_signal_handler as disable_signal_handler
from ..fluid.framework import dygraph_only as dygraph_only
from ..fluid.framework import get_flags as get_flags
from ..fluid.framework import monkey_patch_math_varbase as monkey_patch_math_varbase
from ..fluid.framework import set_flags as set_flags
from ..fluid.layers import monkey_patch_variable as monkey_patch_variable
from ..fluid.layers.tensor import create_parameter as create_parameter
from ..fluid.param_attr import ParamAttr as ParamAttr
from . import random as random
from .framework import get_default_dtype as get_default_dtype
from .framework import is_grad_enabled as is_grad_enabled
from .framework import set_default_dtype as set_default_dtype
from .framework import set_grad_enabled as set_grad_enabled
from .io import load as load
from .io import save as save
from .random import seed as seed

def no_grad(func: Callable[InputArgs, RetValue]) -> Callable[InputArgs, RetValue]: ...
def in_dynamic_mode() -> bool: ...
def enable_static() -> None: ...
def disable_static() -> None: ...



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['paddle-stubs']

package_data = \
{'': ['*'],
 'paddle-stubs': ['amp/*',
                  'autograd/*',
                  'cost_model/*',
                  'dataset/*',
                  'device/*',
                  'device/cuda/*',
                  'distributed/*',
                  'distributed/auto_parallel/*',
                  'distributed/auto_parallel/cost/*',
                  'distributed/auto_parallel/operators/*',
                  'distributed/auto_parallel/tuner/*',
                  'distributed/fleet/*',
                  'distributed/fleet/base/*',
                  'distributed/fleet/data_generator/*',
                  'distributed/fleet/dataset/*',
                  'distributed/fleet/elastic/*',
                  'distributed/fleet/meta_optimizers/*',
                  'distributed/fleet/meta_optimizers/ascend/*',
                  'distributed/fleet/meta_optimizers/dygraph_optimizer/*',
                  'distributed/fleet/meta_optimizers/sharding/*',
                  'distributed/fleet/meta_parallel/*',
                  'distributed/fleet/meta_parallel/parallel_layers/*',
                  'distributed/fleet/meta_parallel/pp_utils/*',
                  'distributed/fleet/meta_parallel/sharding/*',
                  'distributed/fleet/metrics/*',
                  'distributed/fleet/runtime/*',
                  'distributed/fleet/utils/*',
                  'distributed/launch/*',
                  'distributed/launch/context/*',
                  'distributed/launch/controllers/*',
                  'distributed/launch/job/*',
                  'distributed/launch/plugins/*',
                  'distributed/launch/utils/*',
                  'distributed/metric/*',
                  'distributed/models/*',
                  'distributed/models/moe/*',
                  'distributed/passes/*',
                  'distributed/ps/*',
                  'distributed/ps/utils/*',
                  'distributed/sharding/*',
                  'distribution/*',
                  'fluid/*',
                  'fluid/contrib/*',
                  'fluid/contrib/decoder/*',
                  'fluid/contrib/extend_optimizer/*',
                  'fluid/contrib/layers/*',
                  'fluid/contrib/mixed_precision/*',
                  'fluid/contrib/mixed_precision/bf16/*',
                  'fluid/contrib/quantize/*',
                  'fluid/contrib/slim/*',
                  'fluid/contrib/slim/quantization/*',
                  'fluid/contrib/slim/quantization/imperative/*',
                  'fluid/contrib/sparsity/*',
                  'fluid/dataloader/*',
                  'fluid/distributed/*',
                  'fluid/dygraph/*',
                  'fluid/dygraph/amp/*',
                  'fluid/dygraph/dygraph_to_static/*',
                  'fluid/incubate/*',
                  'fluid/incubate/checkpoint/*',
                  'fluid/incubate/data_generator/*',
                  'fluid/incubate/fleet/*',
                  'fluid/incubate/fleet/base/*',
                  'fluid/incubate/fleet/collective/*',
                  'fluid/incubate/fleet/parameter_server/*',
                  'fluid/incubate/fleet/parameter_server/distribute_transpiler/*',
                  'fluid/incubate/fleet/parameter_server/ir/*',
                  'fluid/incubate/fleet/parameter_server/pslib/*',
                  'fluid/incubate/fleet/utils/*',
                  'fluid/inference/*',
                  'fluid/layers/*',
                  'fluid/transpiler/*',
                  'fluid/transpiler/details/*',
                  'framework/*',
                  'hapi/*',
                  'incubate/*',
                  'incubate/autograd/*',
                  'incubate/checkpoint/*',
                  'incubate/multiprocessing/*',
                  'incubate/nn/*',
                  'incubate/nn/functional/*',
                  'incubate/nn/layer/*',
                  'incubate/operators/*',
                  'incubate/optimizer/*',
                  'incubate/optimizer/functional/*',
                  'incubate/passes/*',
                  'incubate/tensor/*',
                  'io/*',
                  'jit/*',
                  'jit/dy2static/*',
                  'libs/*',
                  'metric/*',
                  'nn/*',
                  'nn/functional/*',
                  'nn/initializer/*',
                  'nn/layer/*',
                  'nn/quant/*',
                  'nn/utils/*',
                  'onnx/*',
                  'optimizer/*',
                  'profiler/*',
                  'reader/*',
                  'reader/tests/*',
                  'sparse/*',
                  'sparse/functional/*',
                  'sparse/layer/*',
                  'static/*',
                  'static/amp/*',
                  'static/nn/*',
                  'static/sparsity/*',
                  'tensor/*',
                  'text/*',
                  'text/datasets/*',
                  'utils/*',
                  'utils/cpp_extension/*',
                  'utils/gast/*',
                  'vision/*',
                  'vision/datasets/*',
                  'vision/models/*',
                  'vision/transforms/*']}

install_requires = \
['paddlepaddle>=2.3.1,<3.0.0', 'typing-extensions>=4.3.0,<5.0.0']

setup_kwargs = {
    'name': 'paddlepaddle-stubs',
    'version': '2.3.1.dev1',
    'description': '',
    'long_description': '# PaddlePaddle Stubs <sup>WIP</sup>\n\nA stubs package as described in [PEP 561](https://peps.python.org/pep-0561/) for [PaddlePaddle](https://github.com/PaddlePaddle/Paddle).\n\n<p align="center">\n   <a href="https://python.org/" target="_blank"><img alt="PyPI - Python Version" src="https://img.shields.io/pypi/pyversions/paddlepaddle-stubs?logo=python&style=flat-square"></a>\n   <a href="https://pypi.org/project/paddlepaddle-stubs/" target="_blank"><img src="https://img.shields.io/pypi/v/paddlepaddle-stubs?style=flat-square" alt="pypi"></a>\n   <a href="https://pypi.org/project/paddlepaddle-stubs/" target="_blank"><img alt="PyPI - Downloads" src="https://img.shields.io/pypi/dm/paddlepaddle-stubs?style=flat-square"></a>\n   <a href="LICENSE"><img alt="LICENSE" src="https://img.shields.io/github/license/ShigureLab/paddlepaddle-stubs?style=flat-square"></a>\n   <a href="https://github.com/psf/black"><img alt="black" src="https://img.shields.io/badge/code%20style-black-000000?style=flat-square"></a>\n   <a href="https://gitmoji.dev"><img src="https://img.shields.io/badge/gitmoji-%20😜%20😍-FFDD67?style=flat-square" alt="Gitmoji"></a>\n</p>\n\n目前仅针对 [Pyright](https://github.com/microsoft/pyright) 进行了编写和测试，并没有针对 Mypy 进行编写和测试，因此目前可能仅仅与 VS Code 的 Pylance 扩展一起工作良好～\n\n## Usage\n\n```bash\npip install paddlepaddle-stubs --pre\n```\n\n此时再打开编辑器，查看编辑器的类型提示～～～\n\nVS Code 推荐配置：\n\n```jsonc\n{\n   "python.languageServer": "Pylance",\n   // 现在有一些类型在 strict mode 工作的并不是很好，推荐先使用 basic mode\n   "python.analysis.typeCheckingMode": "basic",\n   "python.analysis.inlayHints.functionReturnTypes": true,\n   "python.analysis.inlayHints.variableTypes": true\n}\n```\n\n## Status\n\n起步阶段，最低支持 Python3.7，目前基于 PaddlePaddle 2.3.1 开发，第一个可用版本应当是 2.3.1 版本或者更晚的 2.4 版本～\n\n## Roadmap\n\nSee [paddlepaddle-stubs 2.3.1 Roadmap](https://github.com/orgs/ShigureLab/projects/1)\n\n### 2.3.1 alpha\n\n> **Note**\n>\n> 此时本 repo 可 public，并发布到 PyPI\n\n-  [x] 通过 basic mode 的类型检查\n-  [ ] 完善全部 P1 级别的类型信息（strict 下工作良好）\n\n### 2.3.1 beta\n\n-  [ ] 完善全部 P2 级别的类型信息（strict 下工作良好）\n-  [ ] [官网示例高层 API 搭建 MNIST 示例代码](./examples/mnist_example.py)在 strict 模式下工作良好\n\n### Future\n\n下一步工作首先保证全部类型 basic 下工作良好\n\n之后应该是一边升级到新版本（2.4.0），一边继续完善 P3、P4、P5 级别的类型信息（strict 下工作良好）\n\n要添加的还有很多，各个 API 类型信息需要仔细填写（修改自动生成的 `Any`，添加返回值类型），有兴趣的小伙伴可以一起来参与呀～\n',
    'author': 'Nyakku Shigure',
    'author_email': 'sigure.qaq@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/ShigureLab/paddlepaddle-stubs',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from __future__ import annotations

from typing import Any, Optional

from paddle.distribution import distribution
from paddle.fluid import core as core
from paddle.fluid.data_feeder import check_dtype as check_dtype
from paddle.fluid.data_feeder import (
    check_variable_and_dtype as check_variable_and_dtype,
)
from paddle.fluid.framework import in_dygraph_mode as in_dygraph_mode
from paddle.fluid.layers import control_flow as control_flow
from paddle.fluid.layers import elementwise_add as elementwise_add
from paddle.fluid.layers import elementwise_div as elementwise_div
from paddle.fluid.layers import elementwise_mul as elementwise_mul
from paddle.fluid.layers import elementwise_sub as elementwise_sub
from paddle.fluid.layers import nn as nn
from paddle.tensor import arange as arange
from paddle.tensor import concat as concat
from paddle.tensor import gather_nd as gather_nd

class Categorical(distribution.Distribution):
    name: Any = ...
    dtype: str = ...
    logits: Any = ...
    def __init__(self, logits: Any, name: Optional[Any] = ...) -> None: ...
    def sample(self, shape: Any): ...
    def kl_divergence(self, other: Any): ...
    def entropy(self): ...
    def probs(self, value: Any): ...
    def log_prob(self, value: Any): ...
